@@ -101,12 +101,13 @@ class MHAttention(nn.Module):
     Multihead attention, with each head being a Linformer Head
     This feeds directly into a feed forward head
     """
-    def __init__(self, input_size, dim, channels, dim_k, nhead, dropout, activation, checkpoint_level, parameter_sharing, E_proj, F_proj, full_attention):
+    def __init__(self, input_size, dim, channels, dim_k, nhead, dropout, activation, checkpoint_level, parameter_sharing, E_proj, F_proj, full_attention, w_o_intermediate_dim=None):
         super(MHAttention, self).__init__()
         self.heads = nn.ModuleList()
         self.input_size = input_size
         self.dim_k = dim_k
         self.checkpoint_level = checkpoint_level
+        self.w_o_intermediate_dim = w_o_intermediate_dim
         if parameter_sharing != "layerwise":
             E_proj = get_linear(input_size, dim_k)
             F_proj = get_linear(input_size, dim_k) if parameter_sharing == "none" or parameter_sharing == "headwise" else E_proj
@@ -125,7 +126,11 @@ class MHAttention(nn.Module):
             self.to_q.append(self.get_linear())
             self.to_k.append(self.get_linear())
             self.to_v.append(self.get_linear())
-        self.w_o = get_linear(dim*nhead, channels)
+        if w_o_intermediate_dim is None:
+            self.w_o = get_linear(dim*nhead, channels)
+        else:
+            self.w_o_1 = get_linear(dim*nhead, w_o_intermediate_dim)
+            self.w_o_2 = get_linear(w_o_intermediate_dim, channels)
         self.activation = get_act(activation)
 
     def forward(self, tensor, **kwargs):
@@ -141,7 +146,11 @@ class MHAttention(nn.Module):
         out = torch.cat(head_outputs, dim=2)
         if self.activation is not None:
             out = self.activation(out)
-        out = self.w_o(out)
+        if self.w_o_intermediate_dim is None:
+            out = self.w_o(out)
+        else:
+            out = self.w_o_1(out)
+            out = self.w_o_2(out)
         return out
 
 class Linformer(nn.Module):
@@ -149,7 +158,7 @@ class Linformer(nn.Module):
     My attempt at reproducing the Linformer Paper
     https://arxiv.org/pdf/2006.04768.pdf
     """
-    def __init__(self, input_size=8192, channels=128, dim_k=64, dim_ff=256, dim_d=None, dropout_ff=0.15, nhead=4, depth=1, dropout=0.1, activation="gelu", use_pos_emb=True, checkpoint_level="C0", parameter_sharing="layerwise", k_reduce_by_layer=0, full_attention=False, include_ff=True):
+    def __init__(self, input_size=8192, channels=128, dim_k=64, dim_ff=256, dim_d=None, dropout_ff=0.15, nhead=4, depth=1, dropout=0.1, activation="gelu", use_pos_emb=True, checkpoint_level="C0", parameter_sharing="layerwise", k_reduce_by_layer=0, full_attention=False, include_ff=True, w_o_intermediate_dim=None):
         super(Linformer, self).__init__()
         assert activation == "gelu" or activation == "relu", "Only gelu and relu activations supported for now"
         assert checkpoint_level == "C0" or checkpoint_level == "C1" or checkpoint_level == "C2", "Checkpoint level has to be either C0, C1, or C2."
@@ -169,7 +178,7 @@ class Linformer(nn.Module):
         self.E = get_linear(input_size, dim_k)
         self.F = self.E
 
-        get_attn = lambda curr_dim_k: MHAttention(input_size, head_dim, channels, curr_dim_k, nhead, dropout, activation, checkpoint_level, parameter_sharing, self.E, self.F, full_attention)
+        get_attn = lambda curr_dim_k: MHAttention(input_size, head_dim, channels, curr_dim_k, nhead, dropout, activation, checkpoint_level, parameter_sharing, self.E, self.F, full_attention, w_o_intermediate_dim)
         get_ff = lambda: FeedForward(channels, dim_ff, dropout_ff)
         get_norm = lambda: nn.LayerNorm(channels)
 
